package com.psmreborn.pkga;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Handler;
import android.util.Log;
import android.view.animation.AlphaAnimation;
import android.widget.ImageView;

import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

public class DownloadImage {
    static ExecutorService pool = Executors.newCachedThreadPool();
    private static final Map<String, ArrayList<imageCallback>> downloadTasks = new HashMap<>();

    private interface imageCallback {
        void run(Bitmap image);
    }

    static imageCallback getImageCallback(String filename, ImageView imageView, Handler handler) {
        return image -> {
            String tag = (String)imageView.getTag();
            if(!filename.equals(tag)) {
                return;
            }
            AlphaAnimation animation = new AlphaAnimation(0f, 1f);
            animation.setDuration(250);
            animation.setFillAfter(true);
            imageView.setAnimation(animation);
            imageView.setImageBitmap(image);
        };
    }

    static void loadImage(Context context, URL url, String filename, ImageView imageView) {
        Handler handler = new Handler(context.getMainLooper());
        imageView.setTag(filename);
        imageView.setImageBitmap(null);

        BitmapFactory.Options bitmapOptions = new BitmapFactory.Options();
        bitmapOptions.outWidth = imageView.getWidth();
        bitmapOptions.outHeight = imageView.getHeight();

        File cachedIcon = new File(context.getCacheDir(), filename);

        synchronized (downloadTasks) {
            if(downloadTasks.containsKey(filename)) {
                downloadTasks.get(filename).add(getImageCallback(filename, imageView, handler));
            } else if(cachedIcon.exists()) {
                final Bitmap icon = BitmapFactory.decodeFile(cachedIcon.getAbsolutePath(), bitmapOptions);
                imageView.setImageBitmap(icon);
            } else {
                ArrayList<imageCallback> callbacks = new ArrayList<>();
                callbacks.add(getImageCallback(filename, imageView, handler));
                downloadTasks.put(filename, callbacks);

                pool.submit(() -> {
                    try {
                        InputStream in = url.openStream();
                        OutputStream out = new FileOutputStream(cachedIcon);

                        byte[] buffer = new byte[1024];
                        int len;
                        while ((len = in.read(buffer)) != -1) {
                            out.write(buffer, 0, len);
                        }
                        out.flush();
                        out.close();
                        in.close();

                        synchronized (downloadTasks) {
                            downloadTasks.remove(filename);
                        }

                        final Bitmap icon = BitmapFactory.decodeFile(cachedIcon.getAbsolutePath(), bitmapOptions);

                        handler.post(() -> {
                            synchronized (downloadTasks) {
                                for (imageCallback callback : callbacks) {
                                    callback.run(icon);
                                }
                            }
                        });
                    } catch (Exception e) {
                        Log.e("Error", e.getMessage());
                        e.printStackTrace();
                    }
                });
            }
        }
    }
}
