package com.psmreborn.pkga;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.Filter;
import android.widget.Filterable;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;

public class GameListAdapter extends ArrayAdapter<Game> implements Filterable {
    public ArrayList<Game> fullGamesList;
    private CharSequence lastSearch = null;
    private final DownloadHandler downloadHandler;

    public GameListAdapter(Context context, DownloadHandler downloadHandler) {
        super(context, R.layout.list_item, new ArrayList<>());
        this.downloadHandler = downloadHandler;
    }

    public interface DownloadHandler {
        void call(Game game);
    }

    @Override
    public View getView(int position, View convertView, ViewGroup parent) {
        Game game = getItem(position);
        assert game != null;
        if (convertView == null) {
            convertView = LayoutInflater.from(getContext()).inflate(R.layout.list_item, parent, false);
        }
        TextView item_title = (TextView)convertView.findViewById(R.id.download_item_desc01);
        Button download_button = (Button)convertView.findViewById(R.id.button_download);
        item_title.setText(game.title);


        // set handler for download button
        download_button.setOnClickListener(view -> {
            this.downloadHandler.call(game);
        });

        ImageView icon = (ImageView)convertView.findViewById(R.id.download_item_image);

        try {
            DownloadImage.loadImage(getContext(), new URL(String.format("http://psmreborn.com/gameinfo/%s/icon_128x128.png", game.titleId)), game.titleId+"_icon.png", icon);
        } catch (MalformedURLException e) {
            throw new RuntimeException(e);
        }

        return convertView;
    }

    public void setGameList(ArrayList<Game> games) {
        fullGamesList = games;
        if(lastSearch != null) {
            this.getFilter().filter(lastSearch);
        } else {
            this.clear();
            for (Game game : games) {
                this.add(game);
            }
            this.notifyDataSetChanged();
        }
    }

    @Override
    public Filter getFilter() {
        return new Filter() {
            @Override
            protected FilterResults performFiltering(CharSequence charSequence) {
                ArrayList<Game> results = new ArrayList<>();
                for (Game game : GameListAdapter.this.fullGamesList) {
                    if(game.title.toLowerCase().contains(charSequence)) {
                        results.add(game);
                    }
                }
                FilterResults filterResults = new FilterResults();
                filterResults.count = results.size();
                filterResults.values = results;
                return filterResults;
            }

            @Override
            protected void publishResults(CharSequence charSequence, FilterResults filterResults) {
                GameListAdapter.this.clear();
                if (filterResults.values instanceof ArrayList) {
                    ArrayList<?> values = (ArrayList<?>) filterResults.values;
                    for (Object game : values) {
                        GameListAdapter.this.add((Game) game);
                    }
                }
                GameListAdapter.this.lastSearch = charSequence;
                GameListAdapter.this.notifyDataSetChanged();
            }
        };
    }
}
