package com.psmreborn.pkga;

import android.Manifest;
import android.app.Activity;
import android.content.Context;
import android.content.DialogInterface;
import android.content.pm.PackageManager;
import android.net.wifi.WifiManager;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.support.v4.app.ActivityCompat;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.animation.Animation;
import android.view.animation.LinearInterpolator;
import android.view.animation.RotateAnimation;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.ListView;

import java.io.File;
import java.util.ArrayList;

import eu.chainfire.libsuperuser.Shell;


public class MainActivity extends Activity {
    private GameListAdapter titleAdapter;

    public void updateGames(ArrayList<Game> newGames){
        this.titleAdapter.setGameList(newGames);
    }

    private void requestPermissionsIfNeeded(String[] permissions) {
        ArrayList<String> needPerms = new ArrayList<String>();
        for(String perm : permissions){
            if(ActivityCompat.checkSelfPermission(this, perm) != PackageManager.PERMISSION_GRANTED){
                needPerms.add(perm);
            }
        }
        if(!needPerms.isEmpty())
            ActivityCompat.requestPermissions(this, needPerms.toArray(new String[0]), 1);
    }

    private void requestPermissions() {
        if(Build.VERSION.SDK_INT >= 23){
            requestPermissionsIfNeeded(new String[] {
                    Manifest.permission.INTERNET,
                    Manifest.permission.WRITE_EXTERNAL_STORAGE,
                    Manifest.permission.ACCESS_WIFI_STATE,
                    Manifest.permission.CHANGE_WIFI_STATE,
                    Manifest.permission.WAKE_LOCK
            });
        }
    }

    private void checkAndStartDownload(Game game){
        new Thread(() -> {
            PsmGameDownloader downloader = new PsmGameDownloader(game);
            Handler handler = new Handler(this.getMainLooper());

            if(downloader.exists()) {
                DialogInterface.OnClickListener clickListener = (dialogInterface, i) -> {
                    if(i == -1) {
                        handler.post(() -> downloader.startDownload(this));
                    }
                };
                handler.post(() -> {
                    Dialogs.getAlertBuilder(this)
                    .setTitle("This Game is already installed!")
                    .setMessage("Press continue to overwrite the current install")
                    .setPositiveButton("Continue", clickListener)
                    .setNegativeButton("Cancel", clickListener)
                    .show();
                });
            } else {
                handler.post(() -> downloader.startDownload(this));
            }
        }).start();
    }
    @Override
    public void onBackPressed(){
        this.finish();
        System.exit(0);

        super.onBackPressed();
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        new Thread(() -> { if(Build.VERSION.SDK_INT >= 30) { Shell.SU.available(); requestPermissions(); } }).start();

        WifiManager wifiManager = (WifiManager) this.getApplicationContext().getSystemService(Context.WIFI_SERVICE);
        if(wifiManager != null){
            wifiManager.setWifiEnabled(true);
        }

        titleAdapter = new GameListAdapter(this.getApplicationContext(), game -> { MainActivity.this.checkAndStartDownload(game); });
        ListView listView = (ListView)this.findViewById(R.id.game_list);
        listView.setAdapter(titleAdapter);

        EditText editText = (EditText) this.findViewById(R.id.search_text);
        editText.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {}

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {
                titleAdapter.getFilter().filter(charSequence);
            }

            @Override
            public void afterTextChanged(Editable editable) {}
        });

        // add refresh button callback
        ImageButton refreshButton = (ImageButton) this.findViewById(R.id.refresh_button);
        refreshButton.setOnClickListener(view -> {
            RotateAnimation rotate = new RotateAnimation(0, 360, Animation.RELATIVE_TO_SELF, 0.5f, Animation.RELATIVE_TO_SELF, 0.5f);
            rotate.setDuration(1000);
            rotate.setRepeatMode(Animation.INFINITE);
            rotate.setInterpolator(new LinearInterpolator());
            refreshButton.setAnimation(rotate);
            MainActivity.this.refreshNps(true, () -> {
                refreshButton.clearAnimation();
            });
        });

        refreshNps(false, () -> { listView.requestFocus(); });
    }

    public void refreshNps(boolean forceRefresh, NoPayStationParser.DoneCallback doneCallback){
        NoPayStationParser nps = new NoPayStationParser(new File(this.getFilesDir(), "PSM_GAMES.tsv"), forceRefresh);
        nps.startParsing(this, doneCallback);
    }
}
