package com.psmreborn.pkga;

import java.io.IOException;
import java.io.InputStream;

public class ProgressInputStream extends InputStream {
    private final InputStream inputStream;
    private final ProgressCallback progressCallback;
    private long totalBytesRead;

    public ProgressInputStream(InputStream inputStream, ProgressCallback progressCallback) {
        this.inputStream = inputStream;
        this.progressCallback = progressCallback;
        this.totalBytesRead = 0;
    }

    @Override
    public int read() throws IOException {
        int data = inputStream.read();
        if (data != -1) {
            totalBytesRead++;
            progressCallback.onProgress(totalBytesRead);
        }
        return data;
    }

    @Override
    public int read(byte[] b) throws IOException {
        int bytesRead = inputStream.read(b);
        if (bytesRead != -1) {
            totalBytesRead += bytesRead;
            progressCallback.onProgress(totalBytesRead);
        }
        return bytesRead;
    }

    @Override
    public int read(byte[] b, int off, int len) throws IOException {
        int bytesRead = inputStream.read(b, off, len);
        if (bytesRead != -1) {
            totalBytesRead += bytesRead;
            progressCallback.onProgress(totalBytesRead);
        }
        return bytesRead;
    }

    @Override
    public void close() throws IOException {
        inputStream.close();
    }

    public interface ProgressCallback {
        void onProgress(long bytesRead);
    }
}
